---
title: llarp::layers::platform::PlatformDriver
summary: base type that is responsible for bridging the os and the flow layer. 

---

# llarp::layers::platform::PlatformDriver



base type that is responsible for bridging the os and the flow layer.  [More...](#detailed-description)


`#include <platform_driver.hpp>`

## Public Functions

|                | Name           |
| -------------- | -------------- |
| | **[PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-platformdriver)**([PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md) * impl, std::shared_ptr< [NetworkConfig](Classes/structllarp_1_1NetworkConfig.md) > netconf, std::shared_ptr< [DnsConfig](Classes/structllarp_1_1DnsConfig.md) > dnsconf) |
| virtual void | **[got_os_traffic](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-got-os-traffic)**(std::vector< [OSTraffic](Classes/structllarp_1_1layers_1_1platform_1_1OSTraffic.md) > pkts)<br>called each time we get a bunch of os traffic.  |
| void | **[wakeup_os](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-wakeup-os)**() const<br>wake up lokinet event loop telling it we sent some things to the network.  |
| void | **[wakeup_flow_layer](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-wakeup-flow-layer)**() const<br>wake up the flow layer. tell it we sent it some things and should process it.  |
| std::shared_ptr< [NameResolverJob_Base](Classes/classllarp_1_1layers_1_1platform_1_1NameResolverJob__Base.md) > | **[make_resolver_job_for](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-make-resolver-job-for)**(UnresolvedName_t name) const<br>given a name we want to look up, make a one shot job that we will use to look it up from the network.  |
| void | **[map_remote](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-map-remote)**(std::string name, std::string auth, std::vector< [IPRange](Classes/structllarp_1_1IPRange.md) > ranges, std::function< void(std::optional< [flow::FlowInfo](Classes/structllarp_1_1layers_1_1flow_1_1FlowInfo.md) >, std::string)> result_handler) |
| void | **[map_exit](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-map-exit)**(std::string name, std::string auth, std::vector< [IPRange](Classes/structllarp_1_1IPRange.md) > ranges, std::function< void(bool, std::string)> result_handler)<br>calls map_remote() for exits  |
| PlatformStats | **[current_stats](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-current-stats)**() const |

## Protected Functions

|                | Name           |
| -------------- | -------------- |
| void | **[on_before_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-on-before-loop-ready)**()<br>called once per event loop tick, before polling for io.  |
| bool | **[on_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-on-loop-ready)**()<br>called once per event loop tick after platform layer event loop is ready to be read.  |
| void | **[after_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-after-loop-ready)**()<br>called once per event loop after we have processed all our events from a read cycle.  |

## Protected Attributes

|                | Name           |
| -------------- | -------------- |
| std::unique_ptr< [PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md) > | **[_impl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--impl)**  |
| [OSTraffic_IO](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO.md) | **[_os_io](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--os-io)**  |
| [FlowTraffic_IO](Classes/classllarp_1_1layers_1_1platform_1_1FlowTraffic__IO.md) | **[_flow_io](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--flow-io)**  |
| std::shared_ptr< [llarp::dns::Server](Classes/classllarp_1_1dns_1_1Server.md) > | **[_dns](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--dns)**  |
| std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > | **[_wakeup_recv](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--wakeup-recv)** <br>called to wake up the loop to read os traffic  |

## Detailed Description

```cpp
class llarp::layers::platform::PlatformDriver;
```

base type that is responsible for bridging the os and the flow layer. 

each kind of ethertype has its own implementation. 

## Public Functions Documentation

### function PlatformDriver

```cpp
PlatformDriver(
    PlatformDriver_pimpl * impl,
    std::shared_ptr< NetworkConfig > netconf,
    std::shared_ptr< DnsConfig > dnsconf
)
```


### function got_os_traffic

```cpp
virtual void got_os_traffic(
    std::vector< OSTraffic > pkts
)
```

called each time we get a bunch of os traffic. 

overrided by each kind of platform 


### function wakeup_os

```cpp
void wakeup_os() const
```

wake up lokinet event loop telling it we sent some things to the network. 

### function wakeup_flow_layer

```cpp
void wakeup_flow_layer() const
```

wake up the flow layer. tell it we sent it some things and should process it. 

### function make_resolver_job_for

```cpp
std::shared_ptr< NameResolverJob_Base > make_resolver_job_for(
    UnresolvedName_t name
) const
```

given a name we want to look up, make a one shot job that we will use to look it up from the network. 

### function map_remote

```cpp
void map_remote(
    std::string name,
    std::string auth,
    std::vector< IPRange > ranges,
    std::function< void(std::optional< flow::FlowInfo >, std::string)> result_handler
)
```


### function map_exit

```cpp
void map_exit(
    std::string name,
    std::string auth,
    std::vector< IPRange > ranges,
    std::function< void(bool, std::string)> result_handler
)
```

calls map_remote() for exits 

### function current_stats

```cpp
PlatformStats current_stats() const
```


## Protected Functions Documentation

### function on_before_loop_ready

```cpp
void on_before_loop_ready()
```

called once per event loop tick, before polling for io. 

### function on_loop_ready

```cpp
bool on_loop_ready()
```

called once per event loop tick after platform layer event loop is ready to be read. 

returns true if we proccessed events and thus need to call [after_loop_ready()](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-after-loop-ready). 


### function after_loop_ready

```cpp
void after_loop_ready()
```

called once per event loop after we have processed all our events from a read cycle. 

## Protected Attributes Documentation

### variable _impl

```cpp
std::unique_ptr< PlatformDriver_pimpl > _impl;
```


### variable _os_io

```cpp
OSTraffic_IO _os_io;
```


### variable _flow_io

```cpp
FlowTraffic_IO _flow_io;
```


### variable _dns

```cpp
std::shared_ptr< llarp::dns::Server > _dns;
```


### variable _wakeup_recv

```cpp
std::shared_ptr< EventLoopWakeup > _wakeup_recv;
```

called to wake up the loop to read os traffic 

-------------------------------

Updated on 2023-02-01 at 02:20:07 +0000