---
title: llarp/layers/platform/deprecated_driver_pimpl.hpp

---

# llarp/layers/platform/deprecated_driver_pimpl.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::platform::NameLookupJob_Deprecated](Classes/classllarp_1_1layers_1_1platform_1_1NameLookupJob__Deprecated.md)**  |
| class | **[llarp::layers::platform::PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md)**  |
| struct | **[llarp::layers::platform::ip_packet_recv_handler](Classes/structllarp_1_1layers_1_1platform_1_1ip__packet__recv__handler.md)**  |
| class | **[llarp::layers::platform::OSTraffic_IO_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO__pimpl.md)**  |
| struct | **[llarp::layers::platform::write_to_flow_layer_handler](Classes/structllarp_1_1layers_1_1platform_1_1write__to__flow__layer__handler.md)**  |
| class | **[llarp::layers::platform::FlowTraffic_IO_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1FlowTraffic__IO__pimpl.md)**  |




## Source code

```cpp
#pragma once

#include "addr_mapper.hpp"
#include "ethertype.hpp"
#include "llarp/config/config.hpp"
#include "llarp/ev/ev.hpp"
#include "llarp/net/ip_packet.hpp"
#include "llarp/util/thread/queue.hpp"
#include "llarp/util/thread/queue_manager.hpp"
#include "platform_addr.hpp"
#include "name_resolver.hpp"
#include "traffic.hpp"
#include "platform_driver.hpp"

#include <functional>
#include <llarp/layers/layers.hpp>

#include <llarp/dns/question.hpp>
#include <llarp/router/router.hpp>

#include <memory>
#include <stdexcept>
#include <oxenc/variant.h>

namespace llarp::layers::platform
{

  class NameLookupJob_Deprecated : public NameResolverJob_Base
  {
    std::shared_ptr<EndpointBase> _endpoint;

   public:
    NameLookupJob_Deprecated(
        std::shared_ptr<EndpointBase> endpoint_,
        std::string_view name_,
        std::chrono::seconds timeout_)
        : NameResolverJob_Base{name_, timeout_}, _endpoint{std::move(endpoint_)}
    {}

    void
    resolve_flow_addr_async(
        std::function<void(std::optional<flow::FlowAddr>)> handle_result) override
    {
      _endpoint->LookupNameAsync(name, [handle_result = std::move(handle_result)](auto maybe_addr) {
        if (not maybe_addr)
        {
          handle_result(std::nullopt);
          return;
        }

        flow::FlowAddr found{var::visit([](auto&& x) { return "{}"_format(x); }, *maybe_addr)};
        handle_result(found);
      });
    }
  };

  class PlatformDriver_pimpl
  {
    const NetworkConfig _netconf;

   public:
    AbstractRouter& router;

    AddrMapper addr_mapper;

    flow::FlowLayer&
    flow_layer()
    {
      return *router.get_layers()->flow;
    }

    std::shared_ptr<NameResolverJob_Base>
    make_resolver_job_for(UnresolvedName_t name)
    {
      if (auto* dns_ptr = std::get_if<dns::Question>(&name))
        throw std::invalid_argument{
            "cannot resolve '{}': icann dns name queries not implemented rn"_format(
                dns_ptr->qname)};

      return std::make_shared<NameLookupJob_Deprecated>(
          router.get_default_endpoint(), std::get<std::string_view>(name), 0s);
    }

    bool
    put_exit_up()
    {
      bool has_exit = router.HasClientExit();
      if (const auto& poker = router.routePoker(); poker and has_exit)
        poker->Up();

      return has_exit;
    }

    void
    put_exit_down()
    {
      if (const auto& poker = router.routePoker())
        poker->Down();
    }

    void
    wakeup_os()
    {
      router.loop()->wakeup();
    }

    void
    wakeup_flow_layer()
    {
      router.TriggerPump();
    }

    void
    before_loop_ready()
    {}

    bool
    loop_ready()
    {
      // todo: implement me
      return false;
    }

    void
    after_loop_ready()
    {
      // todo: implement me
    }

    PlatformDriver_pimpl(AbstractRouter& router_, NetworkConfig conf)
        : _netconf{std::move(conf)}, router{router_}
    {}
  };

  struct ip_packet_recv_handler
  {
    thread::Queue<OSTraffic>& queue;
    EventLoopWakeup& wakeup;

    void
    operator()(net::IPPacket pkt)
    {
      OSTraffic os_pkt{};
      os_pkt.kind = EtherType_t::ip_unicast;
      os_pkt.src.ip = net::ipv6addr_t::from_host(pkt.srcv6());
      os_pkt.dst.ip = net::ipv6addr_t::from_host(pkt.dstv6());
      os_pkt.data = pkt.steal();

      // fixme: this trypushback will drop if we read faster than we are able to procss but this
      // probably never going to happen
      queue.tryPushBack(std::move(os_pkt));
      wakeup.Trigger();
    }
  };

  class OSTraffic_IO_pimpl
  {
    const NetworkConfig _netconf;
    thread::Queue<OSTraffic> _recv_queue{64};
    std::shared_ptr<EventLoopWakeup> _wakeup_recv;

   public:
    std::shared_ptr<vpn::NetworkInterface> netif;

    AbstractRouter& router;

    OSTraffic_IO_pimpl(AbstractRouter& router_, NetworkConfig conf)
        : _netconf{std::move(conf)}, router{router_}
    {
      if (auto* vpn = router.GetVPNPlatform())
        netif = vpn->CreateInterface(_netconf.vpn_interface_info(), &router);

      if (netif)
      {
        loop()->add_network_interface(netif, ip_packet_recv_handler{_recv_queue, _wakeup_recv});
      }
    }

    void
    wakeup_os() const
    {
      router.loop()->wakeup();
    }

    std::vector<OSTraffic>
    read_platform_traffic()
    {
      std::vector<OSTraffic> pkts;
      const bool has_vpn = netif != nullptr;
      while (has_vpn)
      {
        OSTraffic os_pkt{};
        os_pkt.kind = EtherType_t::ip_unicast;

        auto pkt = netif->ReadNextPacket();
        if (pkt.empty())
          break;
        os_pkt.src.ip = net::ipv6addr_t::from_host(pkt.srcv6());
        os_pkt.dst.ip = net::ipv6addr_t::from_host(pkt.dstv6());
        os_pkt.data = pkt.steal();
        pkts.push_back(std::move(os_pkt));
      }
      return pkts;
    }

    void
    write_platform_traffic(std::vector<OSTraffic> traff)
    {
      if (not netif)
        return;
      for (auto& os_pkt : traff)
      {
        net::IPPacket pkt{std::move(os_pkt.data)};
        if (pkt.empty())
          continue;
        if (pkt.IsV4())
          pkt.UpdateIPv4Address(os_pkt.src.as_v4(), os_pkt.dst.as_v4());
        else if (pkt.IsV6())
          pkt.UpdateIPv6Address(ToHost(os_pkt.src.ip), ToHost(os_pkt.dst.ip));
        if (os_pkt.kind == EtherType_t::ip_unicast)
          netif->WritePacket(pkt);
      }
    }
  };

  struct write_to_flow_layer_handler
  {
    std::shared_ptr<EndpointBase> _endpoint;
    std::vector<std::vector<byte_t>> datums;
    service::ProtocolType proto;

    void
    operator()(std::optional<service::ConvoTag> maybe_convotag)
    {
      if (maybe_convotag)
      {
        for (auto& data : datums)
          _endpoint->SendToOrQueue(*maybe_convotag, llarp_buffer_t{data}, proto);
      }
      // TODO: handle failure
    }
  };

  namespace
  {
    EndpointBase::AddressVariant_t
    to_addr_variant(flow::FlowAddr addr)
    {
      switch (addr.kind())
      {
        case flow::FlowAddr::Kind::snode:
          return RouterID{std::move(addr.as_array())};
        case flow::FlowAddr::Kind::snapp:
          return service::Address{std::move(addr.as_array())};
        default:
          throw std::invalid_argument{"bad FlowAddr kind"};
      }
    }
  }  // namespace
  class FlowTraffic_IO_pimpl
  {
   public:
    AbstractRouter& router;
    FlowTraffic_IO_pimpl(AbstractRouter& router_) : router{router_}
    {}

    std::weak_ptr<EndpointBase> endpoint;

    void
    wakeup()
    {
      // todo: is this right? we want to do partial wakeup for just flow layer.
      router.TriggerPump();
    }

    std::vector<flow::Traffic>
    read_from_flow_layer()
    {
      // todo: implement me
      return {};
    }

    // TODO: deprecated implementation cannot use flow tag on the flow info we pass
    void
    write_many_to_flow_layer(
        EtherType_t kind, flow::FlowInfo flow_info, std::vector<std::vector<byte_t>> datums)
    {
      if (kind == EtherType_t::ip_unicast)
      {
        if (auto ptr = endpoint.lock())
        {
          if (not ptr->EnsurePathTo(
                  to_addr_variant(std::move(flow_info.addr)),
                  write_to_flow_layer_handler{
                      ptr, std::move(datums), service::ProtocolType::TrafficV4},
                  2s))
          {
            // todo: handle error?
          }
        }
      }
    }
  };
}  // namespace llarp::layers::platform
```


-------------------------------

Updated on 2023-02-01 at 02:20:08 +0000
