---
title: llarp/layers/platform/platform_driver.hpp

---

# llarp/layers/platform/platform_driver.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::dns](Namespaces/namespacellarp_1_1dns.md)**  |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::platform::OSTraffic_IO](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO.md)**  |
| class | **[llarp::layers::platform::FlowTraffic_IO](Classes/classllarp_1_1layers_1_1platform_1_1FlowTraffic__IO.md)**  |
| class | **[llarp::layers::platform::PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md)** <br>base type that is responsible for bridging the os and the flow layer.  |




## Source code

```cpp
#pragma once

#include "addr_mapper.hpp"
#include "llarp/config/config.hpp"
#include "name_resolver.hpp"
#include "traffic.hpp"

#include <llarp/layers/flow/flow_identity.hpp>
#include <llarp/layers/flow/flow_info.hpp>
#include <llarp/layers/flow/flow_addr.hpp>
#include <llarp/layers/flow/traffic.hpp>

#include <llarp/net/ip_range.hpp>
#include <llarp/util/formattable.hpp>
#include <llarp/util/types.hpp>

#include <memory>
#include <optional>
#include <string>
#include <string_view>
#include <vector>

namespace llarp::dns
{
  class Server;
}

namespace llarp::layers::platform
{

  class OSTraffic_IO_pimpl;

  class OSTraffic_IO
  {
    std::unique_ptr<OSTraffic_IO_pimpl> _impl;

   public:
    OSTraffic_IO(OSTraffic_IO_pimpl*);

    std::vector<OSTraffic>
    read_platform_traffic() const;

    void
    write_platform_traffic(std::vector<OSTraffic>&& traff) const;

    void
    wakeup() const;
  };

  class FlowTraffic_IO_pimpl;

  class FlowTraffic_IO
  {
    std::unique_ptr<FlowTraffic_IO_pimpl> _impl;

   public:
    FlowTraffic_IO(FlowTraffic_IO_pimpl*);

    void
    write_many_to_flow_layer(
        EtherType_t kind, flow::FlowInfo to, std::vector<std::vector<byte_t>> datums);

    void
    write_to_flow_layer(EtherType_t kind, flow::FlowInfo to, std::vector<byte_t> data)
    {
      write_many_to_flow_layer(
          std::move(kind), std::move(to), std::vector<decltype(data)>{std::move(data)});
    }

    std::vector<flow::Traffic>
    read_from_flow_layer();

    void
    wakeup() const;
  };

  class PlatformDriver_pimpl;

  class PlatformDriver
  {
   protected:
    std::unique_ptr<PlatformDriver_pimpl> _impl;
    OSTraffic_IO _os_io;
    FlowTraffic_IO _flow_io;

    std::shared_ptr<llarp::dns::Server> _dns;
    std::shared_ptr<EventLoopWakeup> _wakeup_recv;

    void
    on_before_loop_ready();

    bool
    on_loop_ready();

    void
    after_loop_ready();

   public:
    PlatformDriver(
        PlatformDriver_pimpl* impl,
        std::shared_ptr<NetworkConfig> netconf,
        std::shared_ptr<DnsConfig> dnsconf);

    virtual void
    got_os_traffic(std::vector<OSTraffic> pkts);

    void
    wakeup_os() const;

    void
    wakeup_flow_layer() const;

    std::shared_ptr<NameResolverJob_Base>
    make_resolver_job_for(UnresolvedName_t name) const;

    void
    map_remote(
        std::string name,
        std::string auth,
        std::vector<IPRange> ranges,
        std::function<void(std::optional<flow::FlowInfo>, std::string)> result_handler);

    void
    map_exit(
        std::string name,
        std::string auth,
        std::vector<IPRange> ranges,
        std::function<void(bool, std::string)> result_handler);

    PlatformStats
    current_stats() const;
  };
}  // namespace llarp::layers::platform

namespace llarp
{
  template <>
  inline constexpr bool IsToStringFormattable<layers::platform::PlatformAddr> = true;
}
```


-------------------------------

Updated on 2023-02-01 at 02:20:08 +0000
